@@ -2,11 +2,13 @@ import json
 from collections.abc import AsyncIterator
 from dataclasses import dataclass
 from os import getenv
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Type, Union
+
+from pydantic import BaseModel
 
 from agno.exceptions import ModelProviderError, ModelRateLimitError
 from agno.models.base import Model
-from agno.models.message import Citations, DocumentCitation, Message
+from agno.models.message import Citations, DocumentCitation, Message, UrlCitation
 from agno.models.response import ModelResponse
 from agno.utils.log import log_error, log_warning
 from agno.utils.models.claude import format_messages
@@ -16,6 +18,8 @@ try:
     from anthropic import APIConnectionError, APIStatusError, RateLimitError
     from anthropic import AsyncAnthropic as AsyncAnthropicClient
     from anthropic.types import (
+        CitationPageLocation,
+        CitationsWebSearchResultLocation,
         ContentBlockDeltaEvent,
         ContentBlockStartEvent,
         ContentBlockStopEvent,
@@ -23,7 +27,7 @@ try:
         MessageStopEvent,
     )
     from anthropic.types import Message as AnthropicMessage
-except (ModuleNotFoundError, ImportError):
+except ImportError:
     raise ImportError("`anthropic` not installed. Please install using `pip install anthropic`")
 
 
@@ -74,7 +78,7 @@ class Claude(Model):
         """
         Returns an instance of the Anthropic client.
         """
-        if self.client:
+        if self.client and not self.client.is_closed():
             return self.client
 
         _client_params = self._get_client_params()
@@ -114,7 +118,9 @@ class Claude(Model):
             _request_params.update(self.request_params)
         return _request_params
 
-    def _prepare_request_kwargs(self, system_message: str) -> Dict[str, Any]:
+    def _prepare_request_kwargs(
+        self, system_message: str, tools: Optional[List[Dict[str, Any]]] = None
+    ) -> Dict[str, Any]:
         """
         Prepare the request keyword arguments for the API call.
 
@@ -127,23 +133,25 @@ class Claude(Model):
         request_kwargs = self.request_kwargs.copy()
         request_kwargs["system"] = system_message
 
-        if self._tools:
-            request_kwargs["tools"] = self._format_tools_for_model()
+        if tools:
+            request_kwargs["tools"] = self._format_tools_for_model(tools)
         return request_kwargs
 
-    def _format_tools_for_model(self) -> Optional[List[Dict[str, Any]]]:
+    def _format_tools_for_model(self, tools: Optional[List[Dict[str, Any]]] = None) -> Optional[List[Dict[str, Any]]]:
         """
         Transforms function definitions into a format accepted by the Anthropic API.
-
-        Returns:
-            Optional[List[Dict[str, Any]]]: A list of tools formatted for the API, or None if no functions are defined.
         """
-        if not self._functions:
+        if not tools:
             return None
 
-        tools: List[Dict[str, Any]] = []
-        for func_name, func_def in self._functions.items():
-            parameters: Dict[str, Any] = func_def.parameters or {}
+        parsed_tools: List[Dict[str, Any]] = []
+        for tool_def in tools:
+            if tool_def.get("type", "") != "function":
+                parsed_tools.append(tool_def)
+                continue
+
+            func_def = tool_def.get("function", {})
+            parameters: Dict[str, Any] = func_def.get("parameters", {})
             properties: Dict[str, Any] = parameters.get("properties", {})
             required_params: List[str] = []
 
@@ -165,35 +173,30 @@ class Claude(Model):
                     input_properties[param_name]["type"] = param_info.get("type", "")
 
             tool = {
-                "name": func_name,
-                "description": func_def.description or "",
+                "name": func_def.get("name") or "",
+                "description": func_def.get("description") or "",
                 "input_schema": {
                     "type": parameters.get("type", "object"),
                     "properties": input_properties,
                     "required": required_params,
                 },
             }
-            tools.append(tool)
-        return tools
+            parsed_tools.append(tool)
+        return parsed_tools
 
-    def invoke(self, messages: List[Message]) -> AnthropicMessage:
+    def invoke(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ) -> AnthropicMessage:
         """
         Send a request to the Anthropic API to generate a response.
-
-        Args:
-            messages (List[Message]): A list of messages to send to the model.
-
-        Returns:
-            AnthropicMessage: The response from the model.
-
-        Raises:
-            APIConnectionError: If there are network connectivity issues
-            RateLimitError: If the API rate limit is exceeded
-            APIStatusError: For other API-related errors
         """
         try:
             chat_messages, system_message = format_messages(messages)
-            request_kwargs = self._prepare_request_kwargs(system_message)
+            request_kwargs = self._prepare_request_kwargs(system_message, tools)
 
             return self.get_client().messages.create(
                 model=self.id,
@@ -215,7 +218,13 @@ class Claude(Model):
             log_error(f"Unexpected error calling Claude API: {str(e)}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    def invoke_stream(self, messages: List[Message]) -> Any:
+    def invoke_stream(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ) -> Any:
         """
         Stream a response from the Anthropic API.
 
@@ -224,9 +233,14 @@ class Claude(Model):
 
         Returns:
             Any: The streamed response from the model.
+
+        Raises:
+            APIConnectionError: If there are network connectivity issues
+            RateLimitError: If the API rate limit is exceeded
+            APIStatusError: For other API-related errors
         """
         chat_messages, system_message = format_messages(messages)
-        request_kwargs = self._prepare_request_kwargs(system_message)
+        request_kwargs = self._prepare_request_kwargs(system_message, tools)
 
         try:
             return (
@@ -253,24 +267,19 @@ class Claude(Model):
             log_error(f"Unexpected error calling Claude API: {str(e)}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    async def ainvoke(self, messages: List[Message]) -> AnthropicMessage:
+    async def ainvoke(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ) -> AnthropicMessage:
         """
         Send an asynchronous request to the Anthropic API to generate a response.
-
-        Args:
-            messages (List[Message]): A list of messages to send to the model.
-
-        Returns:
-            AnthropicMessage: The response from the model.
-
-        Raises:
-            APIConnectionError: If there are network connectivity issues
-            RateLimitError: If the API rate limit is exceeded
-            APIStatusError: For other API-related errors
         """
         try:
             chat_messages, system_message = format_messages(messages)
-            request_kwargs = self._prepare_request_kwargs(system_message)
+            request_kwargs = self._prepare_request_kwargs(system_message, tools)
 
             return await self.get_async_client().messages.create(
                 model=self.id,
@@ -292,7 +301,13 @@ class Claude(Model):
             log_error(f"Unexpected error calling Claude API: {str(e)}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    async def ainvoke_stream(self, messages: List[Message]) -> AsyncIterator[Any]:
+    async def ainvoke_stream(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ) -> AsyncIterator[Any]:
         """
         Stream an asynchronous response from the Anthropic API.
 
@@ -301,10 +316,15 @@ class Claude(Model):
 
         Returns:
             Any: The streamed response from the model.
+
+        Raises:
+            APIConnectionError: If there are network connectivity issues
+            RateLimitError: If the API rate limit is exceeded
+            APIStatusError: For other API-related errors
         """
         try:
             chat_messages, system_message = format_messages(messages)
-            request_kwargs = self._prepare_request_kwargs(system_message)
+            request_kwargs = self._prepare_request_kwargs(system_message, tools)
             async with self.get_async_client().messages.stream(
                 model=self.id,
                 messages=chat_messages,  # type: ignore
@@ -350,13 +370,13 @@ class Claude(Model):
                 )
             messages.append(Message(role="user", content=fc_responses))
 
-    def get_system_message_for_model(self) -> Optional[str]:
-        if self._functions is not None and len(self._functions) > 0:
+    def get_system_message_for_model(self, tools: Optional[List[Any]] = None) -> Optional[str]:
+        if tools is not None and len(tools) > 0:
             tool_call_prompt = "Do not reflect on the quality of the returned search results in your response"
             return tool_call_prompt
         return None
 
-    def parse_provider_response(self, response: AnthropicMessage) -> ModelResponse:
+    def parse_provider_response(self, response: AnthropicMessage, **kwargs) -> ModelResponse:
         """
         Parse the Claude response into a ModelResponse.
 
@@ -379,12 +399,25 @@ class Claude(Model):
                     else:
                         model_response.content += block.text
 
-                    if block.citations:
-                        model_response.citations = Citations(raw=block.citations, documents=[])
+                    # Capture citations from the response
+                    if block.citations is not None:
+                        if model_response.citations is None:
+                            model_response.citations = Citations(raw=[], urls=[], documents=[])
                         for citation in block.citations:
-                            model_response.citations.documents.append(  # type: ignore
-                                DocumentCitation(document_title=citation.document_title, cited_text=citation.cited_text)
-                            )
+                            model_response.citations.raw.append(citation.model_dump())  # type: ignore
+                            # Web search citations
+                            if isinstance(citation, CitationsWebSearchResultLocation):
+                                model_response.citations.urls.append(  # type: ignore
+                                    UrlCitation(url=citation.url, title=citation.cited_text)
+                                )
+                            # Document citations
+                            elif isinstance(citation, CitationPageLocation):
+                                model_response.citations.documents.append(  # type: ignore
+                                    DocumentCitation(
+                                        document_title=citation.document_title,
+                                        cited_text=citation.cited_text,
+                                    )
+                                )
                 elif block.type == "thinking":
                     model_response.thinking = block.thinking
                     model_response.provider_data = {
@@ -393,7 +426,7 @@ class Claude(Model):
                 elif block.type == "redacted_thinking":
                     model_response.redacted_thinking = block.data
 
-        # -*- Extract tool calls from the response
+        # Extract tool calls from the response
         if response.stop_reason == "tool_use":
             for block in response.content:
                 if block.type == "tool_use":
@@ -442,12 +475,6 @@ class Claude(Model):
             # Handle text content
             if response.delta.type == "text_delta":
                 model_response.content = response.delta.text
-            elif response.delta.type == "citation_delta":
-                citation = response.delta.citation
-                model_response.citations = Citations(raw=citation)
-                model_response.citations.documents.append(  # type: ignore
-                    DocumentCitation(document_title=citation.document_title, cited_text=citation.cited_text)
-                )
             # Handle thinking content
             elif response.delta.type == "thinking_delta":
                 model_response.thinking = response.delta.thinking
@@ -478,8 +505,24 @@ class Claude(Model):
                     }
                 ]
 
-        # Handle message completion and usage metrics
+        # Capture citations from the final response
         elif isinstance(response, MessageStopEvent):
+            model_response.content = ""
+            model_response.citations = Citations(raw=[], urls=[], documents=[])
+            for block in response.message.content:
+                citations = getattr(block, "citations", None)
+                if not citations:
+                    continue
+                for citation in citations:
+                    model_response.citations.raw.append(citation.model_dump())  # type: ignore
+                    # Web search citations
+                    if isinstance(citation, CitationsWebSearchResultLocation):
+                        model_response.citations.urls.append(UrlCitation(url=citation.url, title=citation.cited_text))  # type: ignore
+                    # Document citations
+                    elif isinstance(citation, CitationPageLocation):
+                        model_response.citations.documents.append(  # type: ignore
+                            DocumentCitation(document_title=citation.document_title, cited_text=citation.cited_text)
+                        )
             if response.message.usage is not None:
                 model_response.response_usage = response.message.usage
 
