@@ -3,7 +3,7 @@ import time
 from dataclasses import dataclass
 from os import getenv
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Type, Union
 from uuid import uuid4
 
 from pydantic import BaseModel
@@ -134,12 +134,14 @@ class Gemini(Model):
         self.client = genai.Client(**client_params)
         return self.client
 
-    def _get_request_kwargs(self, system_message: Optional[str] = None) -> Dict[str, Any]:
+    def _get_request_kwargs(
+        self,
+        system_message: Optional[str] = None,
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+    ) -> Dict[str, Any]:
         """
         Returns the request keyword arguments for the GenerativeModel client.
-
-        Returns:
-            Dict[str, Any]: The request keyword arguments.
         """
         request_params = {}
         # User provides their own generation config
@@ -175,13 +177,9 @@ class Gemini(Model):
         if system_message is not None:
             config["system_instruction"] = system_message  # type: ignore
 
-        if (
-            self.response_format is not None
-            and isinstance(self.response_format, type)
-            and issubclass(self.response_format, BaseModel)
-        ):
+        if response_format is not None and isinstance(response_format, type) and issubclass(response_format, BaseModel):
             config["response_mime_type"] = "application/json"  # type: ignore
-            config["response_schema"] = self.response_format
+            config["response_schema"] = response_format
 
         if self.grounding and self.search:
             log_info("Both grounding and search are enabled. Grounding will take precedence.")
@@ -203,8 +201,8 @@ class Gemini(Model):
             log_info("Search enabled. External tools will be disabled.")
             config["tools"] = [Tool(google_search=GoogleSearch())]
 
-        elif self._tools:
-            config["tools"] = [format_function_definitions(self._tools)]
+        elif tools:
+            config["tools"] = [format_function_definitions(tools)]
 
         config = {k: v for k, v in config.items() if v is not None}
 
@@ -217,18 +215,18 @@ class Gemini(Model):
 
         return request_params
 
-    def invoke(self, messages: List[Message]):
+    def invoke(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ):
         """
         Invokes the model with a list of messages and returns the response.
-
-        Args:
-            messages (List[Message]): The list of messages to send to the model.
-
-        Returns:
-            GenerateContentResponse: The response from the model.
         """
         formatted_messages, system_message = self._format_messages(messages)
-        request_kwargs = self._get_request_kwargs(system_message)
+        request_kwargs = self._get_request_kwargs(system_message, response_format=response_format, tools=tools)
         try:
             return self.get_client().models.generate_content(
                 model=self.id,
@@ -237,26 +235,30 @@ class Gemini(Model):
             )
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
+            error_message = str(e.response) if hasattr(e, "response") else str(e)
             raise ModelProviderError(
-                message=e.response, status_code=e.code, model_name=self.name, model_id=self.id
+                message=error_message,
+                status_code=e.code if hasattr(e, "code") and e.code is not None else 502,
+                model_name=self.name,
+                model_id=self.id,
             ) from e
         except Exception as e:
             log_error(f"Unknown error from Gemini API: {e}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    def invoke_stream(self, messages: List[Message]):
+    def invoke_stream(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ):
         """
         Invokes the model with a list of messages and returns the response as a stream.
-
-        Args:
-            messages (List[Message]): The list of messages to send to the model.
-
-        Returns:
-            Iterator[GenerateContentResponse]: The response from the model as a stream.
         """
         formatted_messages, system_message = self._format_messages(messages)
 
-        request_kwargs = self._get_request_kwargs(system_message)
+        request_kwargs = self._get_request_kwargs(system_message, response_format=response_format, tools=tools)
         try:
             yield from self.get_client().models.generate_content_stream(
                 model=self.id,
@@ -266,19 +268,28 @@ class Gemini(Model):
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
-                message=e.response, status_code=e.code, model_name=self.name, model_id=self.id
+                message=str(e.response) if hasattr(e, "response") else str(e),
+                status_code=e.code if hasattr(e, "code") and e.code is not None else 502,
+                model_name=self.name,
+                model_id=self.id,
             ) from e
         except Exception as e:
             log_error(f"Unknown error from Gemini API: {e}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    async def ainvoke(self, messages: List[Message]):
+    async def ainvoke(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ):
         """
         Invokes the model with a list of messages and returns the response.
         """
         formatted_messages, system_message = self._format_messages(messages)
 
-        request_kwargs = self._get_request_kwargs(system_message)
+        request_kwargs = self._get_request_kwargs(system_message, response_format=response_format, tools=tools)
 
         try:
             return await self.get_client().aio.models.generate_content(
@@ -289,19 +300,28 @@ class Gemini(Model):
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
-                message=e.response, status_code=e.code, model_name=self.name, model_id=self.id
+                message=str(e.response) if hasattr(e, "response") else str(e),
+                status_code=e.code if hasattr(e, "code") and e.code is not None else 502,
+                model_name=self.name,
+                model_id=self.id,
             ) from e
         except Exception as e:
             log_error(f"Unknown error from Gemini API: {e}")
             raise ModelProviderError(message=str(e), model_name=self.name, model_id=self.id) from e
 
-    async def ainvoke_stream(self, messages: List[Message]):
+    async def ainvoke_stream(
+        self,
+        messages: List[Message],
+        response_format: Optional[Union[Dict, Type[BaseModel]]] = None,
+        tools: Optional[List[Dict[str, Any]]] = None,
+        tool_choice: Optional[Union[str, Dict[str, Any]]] = None,
+    ):
         """
         Invokes the model with a list of messages and returns the response as a stream.
         """
         formatted_messages, system_message = self._format_messages(messages)
 
-        request_kwargs = self._get_request_kwargs(system_message)
+        request_kwargs = self._get_request_kwargs(system_message, response_format=response_format, tools=tools)
 
         try:
             async_stream = await self.get_client().aio.models.generate_content_stream(
@@ -314,7 +334,10 @@ class Gemini(Model):
         except (ClientError, ServerError) as e:
             log_error(f"Error from Gemini API: {e}")
             raise ModelProviderError(
-                message=e.response, status_code=e.code, model_name=self.name, model_id=self.id
+                message=str(e.response) if hasattr(e, "response") else str(e),
+                status_code=e.code if hasattr(e, "code") and e.code is not None else 502,
+                model_name=self.name,
+                model_id=self.id,
             ) from e
         except Exception as e:
             log_error(f"Unknown error from Gemini API: {e}")
@@ -328,7 +351,7 @@ class Gemini(Model):
             messages (List[Message]): The list of messages to convert.
         """
         formatted_messages: List = []
-        file_content: Union[GeminiFile, Part] = None
+        file_content: Optional[Union[GeminiFile, Part]] = None
         system_message = None
         for message in messages:
             role = message.role
@@ -385,16 +408,15 @@ class Gemini(Model):
                             # Case 1: Video is a file_types.File object (Recommended)
                             # Add it as a File object
                             if video.content is not None and isinstance(video.content, GeminiFile):
-                                # Google recommends that if using a single image, place the text prompt after the image.
-                                message_parts.insert(
-                                    0, Part.from_uri(file_uri=video.content.uri, mime_type=video.content.mime_type)
-                                )
+                                # Google recommends that if using a single video, place the text prompt after the video.
+                                if video.content.uri and video.content.mime_type:
+                                    message_parts.insert(
+                                        0, Part.from_uri(file_uri=video.content.uri, mime_type=video.content.mime_type)
+                                    )
                             else:
                                 video_file = self._format_video_for_message(video)
-
-                                # Google recommends that if using a single video, place the text prompt after the video.
                                 if video_file is not None:
-                                    message_parts.insert(0, video_file)  # type: ignore
+                                    message_parts.insert(0, video_file)
                     except Exception as e:
                         log_warning(f"Failed to load video from {message.videos}: {e}")
                         continue
@@ -404,13 +426,15 @@ class Gemini(Model):
                     try:
                         for audio_snippet in message.audio:
                             if audio_snippet.content is not None and isinstance(audio_snippet.content, GeminiFile):
-                                # Google recommends that if using a single image, place the text prompt after the image.
-                                message_parts.insert(
-                                    0,
-                                    Part.from_uri(
-                                        file_uri=audio_snippet.content.uri, mime_type=audio_snippet.content.mime_type
-                                    ),
-                                )
+                                # Google recommends that if using a single audio file, place the text prompt after the audio file.
+                                if audio_snippet.content.uri and audio_snippet.content.mime_type:
+                                    message_parts.insert(
+                                        0,
+                                        Part.from_uri(
+                                            file_uri=audio_snippet.content.uri,
+                                            mime_type=audio_snippet.content.mime_type,
+                                        ),
+                                    )
                             else:
                                 audio_content = self._format_audio_for_message(audio_snippet)
                                 if audio_content:
@@ -437,15 +461,13 @@ class Gemini(Model):
     def _format_audio_for_message(self, audio: Audio) -> Optional[Union[Part, GeminiFile]]:
         # Case 1: Audio is a bytes object
         if audio.content and isinstance(audio.content, bytes):
-            return Part.from_bytes(
-                mime_type=f"audio/{audio.format}" if audio.format else "audio/mp3", data=audio.content
-            )
+            mime_type = f"audio/{audio.format}" if audio.format else "audio/mp3"
+            return Part.from_bytes(mime_type=mime_type, data=audio.content)
 
         # Case 2: Audio is an url
-        elif audio.url is not None:
-            return Part.from_bytes(
-                mime_type=f"audio/{audio.format}" if audio.format else "audio/mp3", data=audio.audio_url_content
-            )
+        elif audio.url is not None and audio.audio_url_content is not None:
+            mime_type = f"audio/{audio.format}" if audio.format else "audio/mp3"
+            return Part.from_bytes(mime_type=mime_type, data=audio.audio_url_content)
 
         # Case 3: Audio is a local file path
         elif audio.filepath is not None:
@@ -455,12 +477,12 @@ class Gemini(Model):
             # Check if video is already uploaded
             existing_audio_upload = None
             try:
-                existing_audio_upload = self.get_client().files.get(name=remote_file_name)
+                if remote_file_name:
+                    existing_audio_upload = self.get_client().files.get(name=remote_file_name)
             except Exception as e:
                 log_warning(f"Error getting file {remote_file_name}: {e}")
-                pass
 
-            if existing_audio_upload:
+            if existing_audio_upload and existing_audio_upload.state and existing_audio_upload.state.name == "SUCCESS":
                 audio_file = existing_audio_upload
             else:
                 # Upload the video file to the Gemini API
@@ -475,18 +497,22 @@ class Gemini(Model):
                     )
                 else:
                     log_error(f"Audio file {audio_path} does not exist.")
-                    raise Exception(f"Audio file {audio_path} does not exist.")
+                    return None
 
                 # Check whether the file is ready to be used.
-                while audio_file.state.name == "PROCESSING":
+                while audio_file.state and audio_file.state.name == "PROCESSING":
+                    if audio_file.name:
+                        audio_file = self.get_client().files.get(name=audio_file.name)
                     time.sleep(2)
-                    audio_file = self.get_client().files.get(name=audio_file.name)
 
-                if audio_file.state.name == "FAILED":
-                    raise ValueError(audio_file.state.name)
-            return Part.from_uri(
-                file_uri=audio_file.uri, mime_type=f"audio/{audio.format}" if audio.format else "audio/mp3"
-            )
+                if audio_file.state and audio_file.state.name == "FAILED":
+                    log_error(f"Audio file processing failed: {audio_file.state.name}")
+                    return None
+
+            if audio_file.uri:
+                mime_type = f"audio/{audio.format}" if audio.format else "audio/mp3"
+                return Part.from_uri(file_uri=audio_file.uri, mime_type=mime_type)
+            return None
         else:
             log_warning(f"Unknown audio type: {type(audio.content)}")
             return None
@@ -494,9 +520,8 @@ class Gemini(Model):
     def _format_video_for_message(self, video: Video) -> Optional[Part]:
         # Case 1: Video is a bytes object
         if video.content and isinstance(video.content, bytes):
-            return Part.from_bytes(
-                mime_type=f"video/{video.format}" if video.format else "video/mp4", data=video.content
-            )
+            mime_type = f"video/{video.format}" if video.format else "video/mp4"
+            return Part.from_bytes(mime_type=mime_type, data=video.content)
         # Case 2: Video is stored locally
         elif video.filepath is not None:
             video_path = video.filepath if isinstance(video.filepath, Path) else Path(video.filepath)
@@ -505,12 +530,12 @@ class Gemini(Model):
             # Check if video is already uploaded
             existing_video_upload = None
             try:
-                existing_video_upload = self.get_client().files.get(name=remote_file_name)
+                if remote_file_name:
+                    existing_video_upload = self.get_client().files.get(name=remote_file_name)
             except Exception as e:
                 log_warning(f"Error getting file {remote_file_name}: {e}")
-                pass
 
-            if existing_video_upload:
+            if existing_video_upload and existing_video_upload.state and existing_video_upload.state.name == "SUCCESS":
                 video_file = existing_video_upload
             else:
                 # Upload the video file to the Gemini API
@@ -525,24 +550,28 @@ class Gemini(Model):
                     )
                 else:
                     log_error(f"Video file {video_path} does not exist.")
-                    raise Exception(f"Video file {video_path} does not exist.")
+                    return None
 
                 # Check whether the file is ready to be used.
-                while video_file.state.name == "PROCESSING":
+                while video_file.state and video_file.state.name == "PROCESSING":
+                    if video_file.name:
+                        video_file = self.get_client().files.get(name=video_file.name)
                     time.sleep(2)
-                    video_file = self.get_client().files.get(name=video_file.name)
 
-                if video_file.state.name == "FAILED":
-                    raise ValueError(video_file.state.name)
+                if video_file.state and video_file.state.name == "FAILED":
+                    log_error(f"Video file processing failed: {video_file.state.name}")
+                    return None
 
-            return Part.from_uri(
-                file_uri=video_file.uri, mime_type=f"video/{video.format}" if video.format else "video/mp4"
-            )
+            if video_file.uri:
+                mime_type = f"video/{video.format}" if video.format else "video/mp4"
+                return Part.from_uri(file_uri=video_file.uri, mime_type=mime_type)
+            return None
         # Case 3: Video is a URL
         elif video.url is not None:
+            mime_type = f"video/{video.format}" if video.format else "video/webm"
             return Part.from_uri(
                 file_uri=video.url,
-                mime_type=f"video/{video.format}" if video.format else "video/webm",
+                mime_type=mime_type,
             )
         else:
             log_warning(f"Unknown video type: {type(video.content)}")
@@ -550,7 +579,7 @@ class Gemini(Model):
 
     def _format_file_for_message(self, file: File) -> Optional[Part]:
         # Case 1: File is a bytes object
-        if file.content and isinstance(file.content, bytes):
+        if file.content and isinstance(file.content, bytes) and file.mime_type:
             return Part.from_bytes(mime_type=file.mime_type, data=file.content)
 
         # Case 2: File is a URL
@@ -558,54 +587,58 @@ class Gemini(Model):
             url_content = file.file_url_content
             if url_content is not None:
                 content, mime_type = url_content
-                return Part.from_bytes(mime_type=mime_type, data=content)
-            else:
-                log_warning(f"Failed to download file from {file.url}")
-                return None
+                if mime_type and content:
+                    return Part.from_bytes(mime_type=mime_type, data=content)
+            log_warning(f"Failed to download file from {file.url}")
+            return None
 
         # Case 3: File is a local file path
         elif file.filepath is not None:
             file_path = file.filepath if isinstance(file.filepath, Path) else Path(file.filepath)
             if file_path.exists() and file_path.is_file():
                 if file_path.stat().st_size < 20 * 1024 * 1024:  # 20MB in bytes
-                    if file.mime_type is not None:
-                        return Part.from_bytes(mime_type=file.mime_type, data=file_path.read_bytes())
+                    if file.mime_type:
+                        file_content = file_path.read_bytes()
+                        if file_content:
+                            return Part.from_bytes(mime_type=file.mime_type, data=file_content)
                     else:
                         import mimetypes
 
-                        return Part.from_bytes(
-                            mime_type=mimetypes.guess_type(file_path)[0], data=file_path.read_bytes()
-                        )
+                        mime_type_guess = mimetypes.guess_type(file_path)[0]
+                        if mime_type_guess is not None:
+                            file_content = file_path.read_bytes()
+                            if file_content:
+                                mime_type_str: str = str(mime_type_guess)
+                                return Part.from_bytes(mime_type=mime_type_str, data=file_content)
+                    return None
                 else:
                     clean_file_name = f"files/{file_path.stem.lower().replace('_', '')}"
                     remote_file = None
                     try:
-                        remote_file = self.get_client().files.get(name=clean_file_name)
+                        if clean_file_name:
+                            remote_file = self.get_client().files.get(name=clean_file_name)
                     except Exception as e:
                         log_warning(f"Error getting file {clean_file_name}: {e}")
 
-                    if remote_file is not None:
-                        return Part.from_uri(file_uri=remote_file.uri, mime_type=remote_file.mime_type)
-                    else:
-                        file_upload = self.get_client().files.upload(
-                            file=file_path,
-                            config=dict(
-                                name=clean_file_name,
-                            ),
-                        )
-                        # Check whether the file is ready to be used.
-                        while file_upload.state.name == "PROCESSING":
-                            time.sleep(2)
-                            file_upload = self.get_client().files.get(name=file_upload.name)
-                        if file_upload.state.name == "FAILED":
-                            raise ValueError(file_upload.state.name)
-                        return Part.from_uri(file_uri=file_upload.uri, mime_type=file_upload.mime_type)
+                    if (
+                        remote_file
+                        and remote_file.state
+                        and remote_file.state.name == "SUCCESS"
+                        and remote_file.uri
+                        and remote_file.mime_type
+                    ):
+                        file_uri: str = remote_file.uri
+                        file_mime_type: str = remote_file.mime_type
+                        return Part.from_uri(file_uri=file_uri, mime_type=file_mime_type)
             else:
                 log_error(f"File {file_path} does not exist.")
+            return None
 
         # Case 4: File is a Gemini File object
         elif isinstance(file.external, GeminiFile):
-            return file.external
+            if file.external.uri and file.external.mime_type:
+                return Part.from_uri(file_uri=file.external.uri, mime_type=file.external.mime_type)
+            return None
         return None
 
     def format_function_call_results(
@@ -630,7 +663,7 @@ class Gemini(Model):
                 )
             )
 
-    def parse_provider_response(self, response: GenerateContentResponse) -> ModelResponse:
+    def parse_provider_response(self, response: GenerateContentResponse, **kwargs) -> ModelResponse:
         """
         Parse the OpenAI response into a ModelResponse.
 
@@ -643,40 +676,45 @@ class Gemini(Model):
         model_response = ModelResponse()
 
         # Get response message
-        if response.candidates is not None:
-            response_message: Content = response.candidates[0].content
+        response_message = Content(role="model", parts=[])
+        if response.candidates and response.candidates[0].content:
+            response_message = response.candidates[0].content
 
-            # Add role
-            if response_message.role is not None:
-                model_response.role = self.role_map[response_message.role]
+        # Add role
+        if response_message.role is not None:
+            model_response.role = self.role_map[response_message.role]
 
-            # Add content
-            if response_message.parts is not None:
-                for part in response_message.parts:
-                    # Extract text if present
-                    if hasattr(part, "text") and part.text is not None:
-                        model_response.content = part.text
+        # Add content
+        if response_message.parts is not None:
+            for part in response_message.parts:
+                # Extract text if present
+                if hasattr(part, "text") and part.text is not None:
+                    text_content: Optional[str] = getattr(part, "text")
+                    if isinstance(text_content, str):
+                        model_response.content = text_content
+                    else:
+                        model_response.content = str(text_content) if text_content is not None else ""
 
-                    if hasattr(part, "inline_data") and part.inline_data is not None:
-                        model_response.image = ImageArtifact(
-                            id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
-                        )
+                if hasattr(part, "inline_data") and part.inline_data is not None:
+                    model_response.image = ImageArtifact(
+                        id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
+                    )
 
-                    # Extract function call if present
-                    if hasattr(part, "function_call") and part.function_call is not None:
-                        call_id = part.function_call.id if part.function_call.id else str(uuid4())
-                        tool_call = {
-                            "id": call_id,
-                            "type": "function",
-                            "function": {
-                                "name": part.function_call.name,
-                                "arguments": json.dumps(part.function_call.args)
-                                if part.function_call.args is not None
-                                else "",
-                            },
-                        }
+                # Extract function call if present
+                if hasattr(part, "function_call") and part.function_call is not None:
+                    call_id = part.function_call.id if part.function_call.id else str(uuid4())
+                    tool_call = {
+                        "id": call_id,
+                        "type": "function",
+                        "function": {
+                            "name": part.function_call.name,
+                            "arguments": json.dumps(part.function_call.args)
+                            if part.function_call.args is not None
+                            else "",
+                        },
+                    }
 
-                        model_response.tool_calls.append(tool_call)
+                    model_response.tool_calls.append(tool_call)
 
             if response.candidates and response.candidates[0].grounding_metadata is not None:
                 citations = Citations()
@@ -711,66 +749,70 @@ class Gemini(Model):
     def parse_provider_response_delta(self, response_delta: GenerateContentResponse) -> ModelResponse:
         model_response = ModelResponse()
 
-        response_message: Content = response_delta.candidates[0].content
+        if response_delta.candidates and len(response_delta.candidates) > 0:
+            candidate_content = response_delta.candidates[0].content
+            response_message: Content = Content(role="model", parts=[])
+            if candidate_content is not None:
+                response_message = candidate_content
 
-        # Add role
-        if response_message.role is not None:
-            model_response.role = self.role_map[response_message.role]
+            # Add role
+            if response_message.role is not None:
+                model_response.role = self.role_map[response_message.role]
 
-        if response_message.parts is not None:
-            for part in response_message.parts:
-                # Extract text if present
-                if hasattr(part, "text") and part.text is not None:
-                    model_response.content = part.text
+            if response_message.parts is not None:
+                for part in response_message.parts:
+                    # Extract text if present
+                    if hasattr(part, "text") and part.text is not None:
+                        model_response.content = str(part.text) if part.text is not None else ""
 
-                if hasattr(part, "inline_data") and part.inline_data is not None:
-                    model_response.image = ImageArtifact(
-                        id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
-                    )
+                    if hasattr(part, "inline_data") and part.inline_data is not None:
+                        model_response.image = ImageArtifact(
+                            id=str(uuid4()), content=part.inline_data.data, mime_type=part.inline_data.mime_type
+                        )
 
-                # Extract function call if present
-                if hasattr(part, "function_call") and part.function_call is not None:
-                    call_id = part.function_call.id if part.function_call.id else str(uuid4())
-                    tool_call = {
-                        "id": call_id,
-                        "type": "function",
-                        "function": {
-                            "name": part.function_call.name,
-                            "arguments": json.dumps(part.function_call.args)
-                            if part.function_call.args is not None
-                            else "",
-                        },
-                    }
+                    # Extract function call if present
+                    if hasattr(part, "function_call") and part.function_call is not None:
+                        call_id = part.function_call.id if part.function_call.id else str(uuid4())
+                        tool_call = {
+                            "id": call_id,
+                            "type": "function",
+                            "function": {
+                                "name": part.function_call.name,
+                                "arguments": json.dumps(part.function_call.args)
+                                if part.function_call.args is not None
+                                else "",
+                            },
+                        }
 
-                    model_response.tool_calls.append(tool_call)
+                        model_response.tool_calls.append(tool_call)
 
-        if response_delta.candidates and response_delta.candidates[0].grounding_metadata is not None:
-            citations = Citations()
-            grounding_metadata = response_delta.candidates[0].grounding_metadata.model_dump()
-            citations.raw = grounding_metadata
+            if response_delta.candidates[0].grounding_metadata is not None:
+                citations = Citations()
+                grounding_metadata = response_delta.candidates[0].grounding_metadata.model_dump()
+                citations.raw = grounding_metadata
 
-            # Extract url and title
-            chunks = grounding_metadata.pop("grounding_chunks", None) or []
-            citation_pairs = [
-                (chunk.get("web", {}).get("uri"), chunk.get("web", {}).get("title"))
-                for chunk in chunks
-                if chunk.get("web", {}).get("uri")
-            ]
+                # Extract url and title
+                chunks = grounding_metadata.pop("grounding_chunks", None) or []
+                citation_pairs = [
+                    (chunk.get("web", {}).get("uri"), chunk.get("web", {}).get("title"))
+                    for chunk in chunks
+                    if chunk.get("web", {}).get("uri")
+                ]
 
-            # Create citation objects from filtered pairs
-            citations.urls = [UrlCitation(url=url, title=title) for url, title in citation_pairs]
+                # Create citation objects from filtered pairs
+                citations.urls = [UrlCitation(url=url, title=title) for url, title in citation_pairs]
 
-            model_response.citations = citations
+                model_response.citations = citations
 
-        # Extract usage metadata if present
-        if hasattr(response_delta, "usage_metadata") and response_delta.usage_metadata is not None:
-            usage: GenerateContentResponseUsageMetadata = response_delta.usage_metadata
-            model_response.response_usage = {
-                "input_tokens": usage.prompt_token_count or 0,
-                "output_tokens": usage.candidates_token_count or 0,
-                "total_tokens": usage.total_token_count or 0,
-                "cached_tokens": usage.cached_content_token_count or 0,
-            }
+            # Extract usage metadata if present
+            if hasattr(response_delta, "usage_metadata") and response_delta.usage_metadata is not None:
+                usage: GenerateContentResponseUsageMetadata = response_delta.usage_metadata
+                model_response.response_usage = {
+                    "input_tokens": usage.prompt_token_count or 0,
+                    "output_tokens": usage.candidates_token_count or 0,
+                    "total_tokens": usage.total_token_count or 0,
+                    "cached_tokens": usage.cached_content_token_count or 0,
+                }
 
         return model_response
 
