@@ -1 +1,5 @@
 from agno.models.openrouter.openrouter import OpenRouter
+
+__all__ = [
+    "OpenRouter",
+]
