@@ -12,6 +12,7 @@ class LocalFileSystemTools(Toolkit):
         self,
         target_directory: Optional[str] = None,
         default_extension: str = "txt",
+        **kwargs,
     ):
         """
         Initialize the WriteToLocal toolkit.
@@ -19,7 +20,7 @@ class LocalFileSystemTools(Toolkit):
             target_directory (Optional[str]): Default directory to write files to. Creates if doesn't exist.
             default_extension (str): Default file extension to use if none specified.
         """
-        super().__init__(name="write_to_local")
+        super().__init__(name="write_to_local", **kwargs)
 
         self.target_directory = target_directory or os.getcwd()
         self.default_extension = default_extension.lstrip(".")
