@@ -7,13 +7,14 @@ from typing import Any, Dict, List, Literal, Optional, Union
 
 from agno.tools import Toolkit
 from agno.tools.function import Function
-from agno.utils.log import log_debug, logger
+from agno.utils.log import log_debug, log_warning, logger
 from agno.utils.mcp import get_entrypoint_for_tool
 
 try:
     from mcp import ClientSession, StdioServerParameters
     from mcp.client.sse import sse_client
     from mcp.client.stdio import stdio_client
+    from mcp.client.streamable_http import streamablehttp_client
 except (ImportError, ModuleNotFoundError):
     raise ImportError("`mcp` not installed. Please install using `pip install mcp`")
 
@@ -24,8 +25,19 @@ class SSEClientParams:
 
     url: str
     headers: Optional[Dict[str, Any]] = None
-    timeout: Optional[float] = None
-    sse_read_timeout: Optional[float] = None
+    timeout: Optional[float] = 5
+    sse_read_timeout: Optional[float] = 60 * 5
+
+
+@dataclass
+class StreamableHTTPClientParams:
+    """Parameters for Streamable HTTP client connection."""
+
+    url: str
+    headers: Optional[Dict[str, Any]] = None
+    timeout: Optional[timedelta] = timedelta(seconds=30)
+    sse_read_timeout: Optional[timedelta] = timedelta(seconds=60 * 5)
+    terminate_on_close: Optional[bool] = None
 
 
 class MCPTools(Toolkit):
@@ -36,7 +48,7 @@ class MCPTools(Toolkit):
     Can be used in three ways:
     1. Direct initialization with a ClientSession
     2. As an async context manager with StdioServerParameters
-    3. As an async context manager with SSE endpoints
+    3. As an async context manager with SSE or Streamable HTTP client parameters
     """
 
     def __init__(
@@ -45,8 +57,8 @@ class MCPTools(Toolkit):
         *,
         url: Optional[str] = None,
         env: Optional[dict[str, str]] = None,
-        transport: Literal["stdio", "sse"] = "stdio",
-        server_params: Optional[Union[StdioServerParameters, SSEClientParams]] = None,
+        transport: Literal["stdio", "sse", "streamable-http"] = "stdio",
+        server_params: Optional[Union[StdioServerParameters, SSEClientParams, StreamableHTTPClientParams]] = None,
         session: Optional[ClientSession] = None,
         timeout_seconds: int = 5,
         client=None,
@@ -61,13 +73,13 @@ class MCPTools(Toolkit):
             session: An initialized MCP ClientSession connected to an MCP server
             server_params: Parameters for creating a new session
             command: The command to run to start the server. Should be used in conjunction with env.
-            url: The URL endpoint for SSE connection when transport is "sse".
+            url: The URL endpoint for SSE or Streamable HTTP connection when transport is "sse" or "streamable-http".
             env: The environment variables to pass to the server. Should be used in conjunction with command.
             client: The underlying MCP client (optional, used to prevent garbage collection)
             timeout_seconds: Read timeout in seconds for the MCP client
             include_tools: Optional list of tool names to include (if None, includes all)
             exclude_tools: Optional list of tool names to exclude (if None, excludes none)
-            transport: The transport protocol to use, either "stdio" or "sse"
+            transport: The transport protocol to use, either "stdio" or "sse" or "streamable-http"
         """
         super().__init__(name="MCPTools", **kwargs)
 
@@ -83,6 +95,10 @@ class MCPTools(Toolkit):
                 raise ValueError(
                     "One of 'command' or 'server_params' parameters must be provided when using stdio transport"
                 )
+            if transport == "streamable-http" and url is None:
+                raise ValueError(
+                    "One of 'url' or 'server_params' parameters must be provided when using Streamable HTTP transport"
+                )
 
         # Ensure the received server_params are valid for the given transport
         if server_params is not None:
@@ -96,10 +112,17 @@ class MCPTools(Toolkit):
                     raise ValueError(
                         "If using the stdio transport, server_params must be an instance of StdioServerParameters."
                     )
+            elif transport == "streamable-http":
+                if not isinstance(server_params, StreamableHTTPClientParams):
+                    raise ValueError(
+                        "If using the streamable-http transport, server_params must be an instance of StreamableHTTPClientParams."
+                    )
 
         self.timeout_seconds = timeout_seconds
         self.session: Optional[ClientSession] = session
-        self.server_params: Optional[Union[StdioServerParameters, SSEClientParams]] = server_params
+        self.server_params: Optional[Union[StdioServerParameters, SSEClientParams, StreamableHTTPClientParams]] = (
+            server_params
+        )
         self.transport = transport
         self.url = url
 
@@ -112,7 +135,7 @@ class MCPTools(Toolkit):
         else:
             env = {**environ}
 
-        if command is not None and transport != "sse":
+        if command is not None and transport not in ["sse", "streamable-http"]:
             from shlex import split
 
             parts = split(command)
@@ -136,20 +159,26 @@ class MCPTools(Toolkit):
                 await self.initialize()
             return self
 
-        # Create a new session using stdio_client or sse_client based on transport
+        # Create a new session using stdio_client, sse_client or streamablehttp_client based on transport
         if self.transport == "sse":
-            sse_args = asdict(self.server_params) if self.server_params is not None else {}  # type: ignore
-            if "url" not in sse_args:
-                sse_args["url"] = self.url
-            self._context = sse_client(**sse_args)  # type: ignore
-        else:
+            sse_params = asdict(self.server_params) if self.server_params is not None else {}
+            if "url" not in sse_params:
+                sse_params["url"] = self.url
+            self._context = sse_client(**sse_params)
+        elif self.transport == "streamable-http":
+            streamable_http_params = asdict(self.server_params) if self.server_params is not None else {}
+            if "url" not in streamable_http_params:
+                streamable_http_params["url"] = self.url
+            self._context = streamablehttp_client(**streamable_http_params)
+        elif self.transport == "stdio":
             if self.server_params is None:
                 raise ValueError("server_params must be provided when using stdio transport.")
             self._context = stdio_client(self.server_params)  # type: ignore
 
-        read, write = await self._context.__aenter__()  # type: ignore
+        session_params = await self._context.__aenter__()  # type: ignore
+        read, write = session_params[0:2]
 
-        self._session_context = ClientSession(read, write, read_timeout_seconds=timedelta(self.timeout_seconds))  # type: ignore
+        self._session_context = ClientSession(read, write, read_timeout_seconds=timedelta(seconds=self.timeout_seconds))  # type: ignore
         self.session = await self._session_context.__aenter__()  # type: ignore
 
         # Initialize with the new session
@@ -203,7 +232,6 @@ class MCPTools(Toolkit):
                 try:
                     # Get an entrypoint for the tool
                     entrypoint = get_entrypoint_for_tool(tool, self.session)
-
                     # Create a Function for the tool
                     f = Function(
                         name=tool.name,
@@ -235,16 +263,19 @@ class MultiMCPTools(Toolkit):
     Can be used in three ways:
     1. Direct initialization with a ClientSession
     2. As an async context manager with StdioServerParameters
-    3. As an async context manager with SSE endpoints
+    3. As an async context manager with SSE or Streamable HTTP endpoints
     """
 
     def __init__(
         self,
         commands: Optional[List[str]] = None,
         urls: Optional[List[str]] = None,
+        urls_transports: Optional[List[Literal["sse", "streamable-http"]]] = None,
         *,
         env: Optional[dict[str, str]] = None,
-        server_params_list: Optional[List[Union[SSEClientParams, StdioServerParameters]]] = None,
+        server_params_list: Optional[
+            List[Union[SSEClientParams, StdioServerParameters, StreamableHTTPClientParams]]
+        ] = None,
         timeout_seconds: int = 5,
         client=None,
         include_tools: Optional[list[str]] = None,
@@ -256,8 +287,9 @@ class MultiMCPTools(Toolkit):
 
         Args:
             commands: List of commands to run to start the servers. Should be used in conjunction with env.
-            urls: List of URLs for SSE endpoints.
-            server_params_list: List of StdioServerParameters or SSEClientParams for creating new sessions.
+            urls: List of URLs for SSE and/or Streamable HTTP endpoints.
+            urls_transports: List of transports to use for the given URLs.
+            server_params_list: List of StdioServerParameters or SSEClientParams or StreamableHTTPClientParams for creating new sessions.
             env: The environment variables to pass to the servers. Should be used in conjunction with commands.
             client: The underlying MCP client (optional, used to prevent garbage collection).
             timeout_seconds: Timeout in seconds for managing timeouts for Client Session if Agent or Tool doesn't respond.
@@ -266,6 +298,15 @@ class MultiMCPTools(Toolkit):
         """
         super().__init__(name="MultiMCPTools", **kwargs)
 
+        if urls is not None:
+            if urls_transports is None:
+                log_warning(
+                    "The default transport 'sse' will be used. You can explicitly set the transports by providing the urls_transports parameter."
+                )
+            else:
+                if len(urls) != len(urls_transports):
+                    raise ValueError("urls and urls_transports must be of the same length")
+
         # Set these after `__init__` to bypass the `_check_tools_filters`
         # beacuse tools are not available until `initialize()` is called.
         self.include_tools = include_tools
@@ -274,11 +315,12 @@ class MultiMCPTools(Toolkit):
         if server_params_list is None and commands is None and urls is None:
             raise ValueError("Either server_params_list or commands or urls must be provided")
 
-        self.server_params_list: List[Union[SSEClientParams, StdioServerParameters]] = server_params_list or []
+        self.server_params_list: List[Union[SSEClientParams, StdioServerParameters, StreamableHTTPClientParams]] = (
+            server_params_list or []
+        )
         self.timeout_seconds = timeout_seconds
         self.commands: Optional[List[str]] = commands
         self.urls: Optional[List[str]] = urls
-
         # Merge provided env with system env
         if env is not None:
             env = {
@@ -300,8 +342,15 @@ class MultiMCPTools(Toolkit):
                 self.server_params_list.append(StdioServerParameters(command=cmd, args=arguments, env=env))
 
         if urls is not None:
-            for url in urls:
-                self.server_params_list.append(SSEClientParams(url=url))
+            if urls_transports is not None:
+                for url, transport in zip(urls, urls_transports):
+                    if transport == "streamable-http":
+                        self.server_params_list.append(StreamableHTTPClientParams(url=url))
+                    else:
+                        self.server_params_list.append(SSEClientParams(url=url))
+            else:
+                for url in urls:
+                    self.server_params_list.append(SSEClientParams(url=url))
 
         self._async_exit_stack = AsyncExitStack()
 
@@ -316,18 +365,25 @@ class MultiMCPTools(Toolkit):
                 stdio_transport = await self._async_exit_stack.enter_async_context(stdio_client(server_params))
                 read, write = stdio_transport
                 session = await self._async_exit_stack.enter_async_context(
-                    ClientSession(read, write, read_timeout_seconds=timedelta(self.timeout_seconds))
+                    ClientSession(read, write, read_timeout_seconds=timedelta(seconds=self.timeout_seconds))
                 )
-
+                await self.initialize(session)
+            # Handle SSE connections
+            elif isinstance(server_params, SSEClientParams):
+                client_connection = await self._async_exit_stack.enter_async_context(
+                    sse_client(**asdict(server_params))
+                )
+                read, write = client_connection
+                session = await self._async_exit_stack.enter_async_context(ClientSession(read, write))
                 await self.initialize(session)
 
-            # Handle SSE connections
-            if isinstance(server_params, SSEClientParams):
-                sse_args = asdict(server_params)
-                sse_transport = await self._async_exit_stack.enter_async_context(sse_client(**sse_args))
-                read, write = sse_transport
+            # Handle Streamable HTTP connections
+            elif isinstance(server_params, StreamableHTTPClientParams):
+                client_connection = await self._async_exit_stack.enter_async_context(
+                    streamablehttp_client(**asdict(server_params))
+                )
+                read, write = client_connection[0:2]
                 session = await self._async_exit_stack.enter_async_context(ClientSession(read, write))
-
                 await self.initialize(session)
 
         return self
