@@ -32,6 +32,9 @@ class OpenAITools(Toolkit):
         text_to_speech_model: OpenAITTSModel = "tts-1",
         text_to_speech_format: OpenAITTSFormat = "mp3",
         image_model: Optional[str] = "dall-e-3",
+        image_quality: Optional[str] = None,
+        image_size: Optional[Literal["256x256", "512x512", "1024x1024", "1792x1024", "1024x1792"]] = None,
+        image_style: Optional[Literal["vivid", "natural"]] = None,
         **kwargs,
     ):
         super().__init__(name="openai_tools", **kwargs)
@@ -46,6 +49,9 @@ class OpenAITools(Toolkit):
         self.tts_model = text_to_speech_model
         self.tts_format = text_to_speech_format
         self.image_model = image_model
+        self.image_quality = image_quality
+        self.image_style = image_style
+        self.image_size = image_size
 
         if enable_transcription:
             self.register(self.transcribe_audio)
@@ -85,29 +91,50 @@ class OpenAITools(Toolkit):
             prompt (str): The text prompt to generate the image from.
         """
         try:
-            response = OpenAIClient().images.generate(
-                model=self.image_model,
-                prompt=prompt,
-                response_format="url",
-            )
+            extra_params = {
+                "size": self.image_size,
+                "quality": self.image_quality,
+                "style": self.image_style,
+            }
+            extra_params = {k: v for k, v in extra_params.items() if v is not None}
 
-            if response.data and response.data[0].url:
-                image_url = response.data[0].url
+            # gpt-image-1 by default outputs a base64 encoded image but other models do not
+            # so we add a response_format parameter to have consistent output.
+            if self.image_model and self.image_model.startswith("gpt-image"):
+                response = OpenAIClient().images.generate(
+                    model=self.image_model,
+                    prompt=prompt,
+                    **extra_params,
+                )
+            else:
+                response = OpenAIClient().images.generate(
+                    model=self.image_model,
+                    prompt=prompt,
+                    response_format="b64_json",
+                    **extra_params,
+                )
+            data = None
+            if hasattr(response, "data") and response.data:
+                data = response.data[0]
+            if data is None:
+                log_warning("OpenAI API did not return any data.")
+                return "Failed to generate image: No data received from API."
+            if hasattr(data, "b64_json") and data.b64_json:
+                image_base64 = data.b64_json
                 media_id = str(uuid4())
+                # Store base64-encoded content as bytes for later saving
                 agent.add_image(
                     ImageArtifact(
                         id=media_id,
-                        url=image_url,
+                        content=image_base64.encode("utf-8"),
+                        mime_type="image/png",
                     )
                 )
-                return f"Image generated successfully: {image_url}"
-            else:
-                log_warning("OpenAI API did not return an image URL.")
-                return "Failed to generate image: No URL received from API."
-
+                return "Image generated successfully."
+            return "Failed to generate image: No content received from API."
         except Exception as e:
-            log_error(f"Failed to generate image using {self.image_model}: {str(e)}")
-            return f"Failed to generate image: {str(e)}"
+            log_error(f"Failed to generate image using {self.image_model}: {e}")
+            return f"Failed to generate image: {e}"
 
     def generate_speech(
         self,
