@@ -49,6 +49,8 @@ class SQLTools(Toolkit):
         self.db_engine: Engine = _engine
         self.Session: sessionmaker[Session] = sessionmaker(bind=self.db_engine)
 
+        self.schema = schema
+
         # Tables this toolkit can access
         self.tables: Optional[Dict[str, Any]] = tables
 
@@ -71,7 +73,11 @@ class SQLTools(Toolkit):
 
         try:
             log_debug("listing tables in the database")
-            table_names = inspect(self.db_engine).get_table_names()
+            inspector = inspect(self.db_engine)
+            if self.schema:
+                table_names = inspector.get_table_names(schema=self.schema)
+            else:
+                table_names = inspector.get_table_names()
             log_debug(f"table_names: {table_names}")
             return json.dumps(table_names)
         except Exception as e:
@@ -90,9 +96,14 @@ class SQLTools(Toolkit):
 
         try:
             log_debug(f"Describing table: {table_name}")
-            table_names = inspect(self.db_engine)
-            table_schema = table_names.get_columns(table_name)
-            return json.dumps([str(column) for column in table_schema])
+            inspector = inspect(self.db_engine)
+            table_schema = inspector.get_columns(table_name, schema=self.schema)
+            return json.dumps(
+                [
+                    {"name": column["name"], "type": str(column["type"]), "nullable": column["nullable"]}
+                    for column in table_schema
+                ]
+            )
         except Exception as e:
             logger.error(f"Error getting table schema: {e}")
             return f"Error getting table schema: {e}"
