@@ -1,11 +1,11 @@
 from pathlib import Path
 from typing import Dict, Optional
 
-from agno.utils.log import logger
+from agno.utils.log import log_debug, logger
 
 
 def read_pyproject_agno(pyproject_file: Path) -> Optional[Dict]:
-    logger.debug(f"Reading {pyproject_file}")
+    log_debug(f"Reading {pyproject_file}")
     try:
         import tomli
 
