@@ -2,3 +2,11 @@ from agno.vectordb.distance import Distance
 from agno.vectordb.pgvector.index import HNSW, Ivfflat
 from agno.vectordb.pgvector.pgvector import PgVector
 from agno.vectordb.search import SearchType
+
+__all__ = [
+    "Distance",
+    "HNSW",
+    "Ivfflat",
+    "PgVector",
+    "SearchType",
+]
