@@ -233,16 +233,27 @@ class Qdrant(VectorDb):
             document.embed(embedder=self.embedder)
             cleaned_content = document.content.replace("\x00", "\ufffd")
             doc_id = md5(cleaned_content.encode()).hexdigest()
+
+            # Create payload with document properties
+            payload = {
+                "name": document.name,
+                "meta_data": document.meta_data,
+                "content": cleaned_content,
+                "usage": document.usage,
+            }
+
+            # Add filters as metadata if provided
+            if filters:
+                # Merge filters with existing metadata
+                if "meta_data" not in payload:
+                    payload["meta_data"] = {}
+                payload["meta_data"].update(filters)  # type: ignore
+
             points.append(
                 models.PointStruct(
                     id=doc_id,
                     vector=document.embedding,
-                    payload={
-                        "name": document.name,
-                        "meta_data": document.meta_data,
-                        "content": cleaned_content,
-                        "usage": document.usage,
-                    },
+                    payload=payload,
                 )
             )
             log_debug(f"Inserted document: {document.name} ({document.meta_data})")
@@ -251,23 +262,40 @@ class Qdrant(VectorDb):
         log_debug(f"Upsert {len(points)} documents")
 
     async def async_insert(self, documents: List[Document], filters: Optional[Dict[str, Any]] = None) -> None:
-        """Insert documents asynchronously."""
+        """
+        Insert documents asynchronously.
+
+        Args:
+            documents (List[Document]): List of documents to insert
+            filters (Optional[Dict[str, Any]]): Filters to apply while inserting documents
+        """
         log_debug(f"Inserting {len(documents)} documents asynchronously")
 
         async def process_document(document):
             document.embed(embedder=self.embedder)
             cleaned_content = document.content.replace("\x00", "\ufffd")
             doc_id = md5(cleaned_content.encode()).hexdigest()
+
+            # Create payload with document properties
+            payload = {
+                "name": document.name,
+                "meta_data": document.meta_data,
+                "content": cleaned_content,
+                "usage": document.usage,
+            }
+
+            # Add filters as metadata if provided
+            if filters:
+                # Merge filters with existing metadata
+                if "meta_data" not in payload:
+                    payload["meta_data"] = {}
+                payload["meta_data"].update(filters)
+
             log_debug(f"Inserted document asynchronously: {document.name} ({document.meta_data})")
             return models.PointStruct(
                 id=doc_id,
                 vector=document.embedding,
-                payload={
-                    "name": document.name,
-                    "meta_data": document.meta_data,
-                    "content": cleaned_content,
-                    "usage": document.usage,
-                },
+                payload=payload,
             )
 
         import asyncio
@@ -309,13 +337,45 @@ class Qdrant(VectorDb):
             logger.error(f"Error getting embedding for Query: {query}")
             return []
 
-        results = self.client.search(
-            collection_name=self.collection,
-            query_vector=query_embedding,
-            with_vectors=True,
-            with_payload=True,
-            limit=limit,
-        )
+        # Build search parameters
+        search_params = {
+            "collection_name": self.collection,
+            "query_vector": query_embedding,
+            "with_vectors": True,
+            "with_payload": True,
+            "limit": limit,
+        }
+
+        # Handle filters if provided - normalize format for Qdrant
+        if filters:
+            filter_conditions = []
+            for key, value in filters.items():
+                # If key contains a dot already, assume it's in the correct format
+                # Otherwise, assume it's a metadata field and add the prefix
+                if "." not in key and not key.startswith("meta_data."):
+                    # This is a simple field name, assume it's metadata
+                    key = f"meta_data.{key}"
+
+                if isinstance(value, dict):
+                    # Handle nested dictionaries
+                    for sub_key, sub_value in value.items():
+                        filter_conditions.append(
+                            models.FieldCondition(key=f"{key}.{sub_key}", match=models.MatchValue(value=sub_value))
+                        )
+                else:
+                    # Handle direct key-value pairs
+                    filter_conditions.append(models.FieldCondition(key=key, match=models.MatchValue(value=value)))
+
+            if filter_conditions:
+                # Use query_filter instead of filter
+                search_params["query_filter"] = models.Filter(must=filter_conditions)
+
+        try:
+            # Execute search with parameters
+            results = self.client.search(**search_params)  # type: ignore
+        except Exception as e:
+            logger.error(f"Error searching for documents: {e}")
+            return []
 
         # Build search results
         search_results: List[Document] = []
@@ -336,6 +396,7 @@ class Qdrant(VectorDb):
         if self.reranker:
             search_results = self.reranker.rerank(query=query, documents=search_results)
 
+        log_info(f"Found {len(search_results)} documents")
         return search_results
 
     async def async_search(
@@ -347,13 +408,45 @@ class Qdrant(VectorDb):
             logger.error(f"Error getting embedding for Query: {query}")
             return []
 
-        results = await self.async_client.search(
-            collection_name=self.collection,
-            query_vector=query_embedding,
-            with_vectors=True,
-            with_payload=True,
-            limit=limit,
-        )
+        # Build search parameters
+        search_params = {
+            "collection_name": self.collection,
+            "query_vector": query_embedding,
+            "with_vectors": True,
+            "with_payload": True,
+            "limit": limit,
+        }
+
+        # Handle filters if provided - normalize format for Qdrant
+        if filters:
+            filter_conditions = []
+            for key, value in filters.items():
+                # If key contains a dot already, assume it's in the correct format
+                # Otherwise, assume it's a metadata field and add the prefix
+                if "." not in key and not key.startswith("meta_data."):
+                    # This is a simple field name, assume it's metadata
+                    key = f"meta_data.{key}"
+
+                if isinstance(value, dict):
+                    # Handle nested dictionaries
+                    for sub_key, sub_value in value.items():
+                        filter_conditions.append(
+                            models.FieldCondition(key=f"{key}.{sub_key}", match=models.MatchValue(value=sub_value))
+                        )
+                else:
+                    # Handle direct key-value pairs
+                    filter_conditions.append(models.FieldCondition(key=key, match=models.MatchValue(value=value)))
+
+            if filter_conditions:
+                # Use query_filter instead of filter
+                search_params["query_filter"] = models.Filter(must=filter_conditions)
+
+        try:
+            # Execute search with parameters
+            results = await self.async_client.search(**search_params)  # type: ignore
+        except Exception as e:
+            logger.error(f"Error searching for documents: {e}")
+            return []
 
         # Build search results
         search_results: List[Document] = []
@@ -374,6 +467,7 @@ class Qdrant(VectorDb):
         if self.reranker:
             search_results = self.reranker.rerank(query=query, documents=search_results)
 
+        log_info(f"Found {len(search_results)} documents")
         return search_results
 
     def drop(self) -> None:
